@@ -1,9 +1,9 @@
 /*
- * $Id: dialog.c,v 1.240 2015/09/20 23:48:54 tom Exp $
+ * $Id: dialog.c,v 1.247 2016/04/19 10:55:38 tom Exp $
  *
  *  cdialog - Display simple dialog boxes from shell scripts
  *
- *  Copyright 2000-2014,2015	Thomas E. Dickey
+ *  Copyright 2000-2015,2016	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -151,6 +151,7 @@ typedef enum {
     ,o_editbox
     ,o_fselect
     ,o_timebox
+    ,o_week_start
 #endif
 #ifdef HAVE_XDIALOG2
     ,o_buildlist
@@ -164,6 +165,7 @@ typedef enum {
 #ifdef HAVE_DLG_TRACE
     ,o_trace
 #endif
+    ,o_iso_week
 } eOptions;
 
 /*
@@ -189,6 +191,9 @@ typedef struct {
 static bool *dialog_opts;
 static char **dialog_argv;
 
+static char **special_argv = 0;
+static int special_argc = 0;
+
 static bool ignore_unknown = FALSE;
 
 static const char *program = "dialog";
@@ -320,9 +325,11 @@ static const Options options[] = {
     { "editbox",	o_editbox,		2, "<file> <height> <width>" },
     { "fselect",	o_fselect,		2, "<filepath> <height> <width>" },
     { "timebox",	o_timebox,		2, "<text> <height> <width> <hour> <minute> <second>" },
+    { "week-start",	o_week_start,		1, "<str>" },
+    { "iso-week",	o_iso_week,		1, NULL },
 #endif
 #ifdef HAVE_XDIALOG2
-    { "buildlist",	o_buildlist,		2, "<text> <height> <width> <tag1> <item1> <status1>..." },
+    { "buildlist",	o_buildlist,		2, "<text> <height> <width> <list-height> <tag1> <item1> <status1>..." },
     { "no-items", 	o_no_items,		1, "" },
     { "no-tags", 	o_no_tags,		1, "" },
     { "rangebox",	o_rangebox,		2, "<text> <height> <width> <min-value> <max-value> <default-value>" },
@@ -1294,7 +1301,7 @@ Help(void)
     static const char *const tbl_1[] =
     {
 	"cdialog (ComeOn Dialog!) version %s",
-	"Copyright 2000-2014,2015 Thomas E. Dickey",
+	"Copyright 2000-2015,2016 Thomas E. Dickey",
 	"This is free software; see the source for copying conditions.  There is NO",
 	"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.",
 	"",
@@ -1650,6 +1657,18 @@ process_common_options(int argc, char **argv, int offset, bool output)
 	    dialog_vars.no_tags = TRUE;
 	    break;
 #endif
+#ifdef HAVE_XDIALOG
+	case o_week_start:
+	    dialog_vars.week_start = optionString(argv, &offset);
+	    break;
+#endif
+	case o_iso_week:
+	    dialog_vars.iso_week = TRUE;
+	    if (dialog_vars.week_start == 0) {	/* Monday is implied */
+		static char default_1st[] = "1";
+		dialog_vars.week_start = default_1st;
+	    }
+	    break;
 	}
 	if (!done)
 	    offset++;
@@ -1665,8 +1684,6 @@ static void
 init_result(char *buffer)
 {
     static bool first = TRUE;
-    static char **special_argv = 0;
-    static int special_argc = 0;
 
     dlg_trace_msg("# init_result\n");
 
@@ -1693,19 +1710,8 @@ init_result(char *buffer)
 	first = FALSE;
     }
 
-    /*
-     * If we are not checking memory leaks, just do the parse of the
-     * environment once.
-     */
     if (special_argv != 0) {
 	process_common_options(special_argc, special_argv, 0, FALSE);
-#ifdef NO_LEAKS
-	free(special_argv[0]);
-	free(special_argv);
-	special_argv = 0;
-	special_argc = 0;
-	first = TRUE;
-#endif
     }
 }
 
@@ -2000,5 +2006,13 @@ main(int argc, char *argv[])
 	(void) refresh();
 	end_dialog();
     }
+#ifdef NO_LEAKS
+    if (special_argv != 0) {
+	free(special_argv[0]);
+	free(special_argv);
+	special_argv = 0;
+	special_argc = 0;
+    }
+#endif
     dlg_exit(retval);
 }
