@@ -1,9 +1,9 @@
 /*
- *  $Id: dlg_keys.c,v 1.22 2007/02/22 22:01:23 tom Exp $
+ *  $Id: dlg_keys.c,v 1.41 2016/08/28 19:36:18 tom Exp $
  *
- * dlg_keys.c -- runtime binding support for dialog
+ *  dlg_keys.c -- runtime binding support for dialog
  *
- * Copyright 2006,2007 Thomas E. Dickey
+ *  Copyright 2006-2014,2016 Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -26,6 +26,10 @@
 
 #define LIST_BINDINGS struct _list_bindings
 
+#define CHR_BACKSLASH   '\\'
+#define IsOctal(ch)     ((ch) >= '0' && (ch) <= '7')
+#define TableSize(name) (sizeof(name)/sizeof(name[0]))
+
 LIST_BINDINGS {
     LIST_BINDINGS *link;
     WINDOW *win;		/* window on which widget gets input */
@@ -34,6 +38,7 @@ LIST_BINDINGS {
     DLG_KEYS_BINDING *binding;	/* list of bindings */
 };
 
+#define WILDNAME "*"
 static LIST_BINDINGS *all_bindings;
 static const DLG_KEYS_BINDING end_keys_binding = END_KEYS_BINDING;
 
@@ -61,6 +66,17 @@ dlg_register_window(WINDOW *win, const char *name, DLG_KEYS_BINDING * binding)
 	else
 	    all_bindings = p;
     }
+#if defined(HAVE_DLG_TRACE) && defined(HAVE_RC_FILE)
+    /*
+     * Trace the binding information assigned to this window.  For most widgets
+     * there is only one binding table.  forms have two, so the trace will be
+     * longer.  Since compiled-in bindings are only visible when the widget is
+     * registered, there is no other way to see what bindings are available,
+     * than by running dialog and tracing it.
+     */
+    dlg_trace_msg("# dlg_register_window %s\n", name);
+    dlg_dump_window_keys(dialog_state.trace_output, win);
+#endif
 }
 
 /*
@@ -189,7 +205,7 @@ int
 dlg_lookup_key(WINDOW *win, int curses_key, int *fkey)
 {
     LIST_BINDINGS *p;
-    int n;
+    DLG_KEYS_BINDING *q;
 
     /*
      * Ignore mouse clicks, since they are already encoded properly.
@@ -208,19 +224,30 @@ dlg_lookup_key(WINDOW *win, int curses_key, int *fkey)
     } else
 #endif
     if (*fkey == 0 || curses_key < KEY_MAX) {
+	const char *name = WILDNAME;
+	if (win != 0) {
+	    for (p = all_bindings; p != 0; p = p->link) {
+		if (p->win == win) {
+		    name = p->name;
+		    break;
+		}
+	    }
+	}
 	for (p = all_bindings; p != 0; p = p->link) {
-	    if (p->win == win || p->win == 0) {
+	    if (p->win == win ||
+		(p->win == 0 &&
+		 (!strcmp(p->name, name) || !strcmp(p->name, WILDNAME)))) {
 		int function_key = (*fkey != 0);
-		for (n = 0; p->binding[n].is_function_key >= 0; ++n) {
+		for (q = p->binding; q->is_function_key >= 0; ++q) {
 		    if (p->buttons
 			&& !function_key
-			&& p->binding[n].curses_key == (int) dlg_toupper(curses_key)) {
+			&& q->curses_key == (int) dlg_toupper(curses_key)) {
 			*fkey = 0;
-			return p->binding[n].dialog_key;
+			return q->dialog_key;
 		    }
-		    if (p->binding[n].curses_key == curses_key
-			&& p->binding[n].is_function_key == function_key) {
-			*fkey = p->binding[n].dialog_key;
+		    if (q->curses_key == curses_key
+			&& q->is_function_key == function_key) {
+			*fkey = q->dialog_key;
 			return *fkey;
 		    }
 		}
@@ -281,7 +308,11 @@ dlg_result_key(int dialog_key, int fkey GCC_UNUSED, int *resultp)
     if (dialog_key == ESC) {
 	*resultp = DLG_EXIT_ESC;
 	done = TRUE;
+    } else if (dialog_key == ERR) {
+	*resultp = DLG_EXIT_ERROR;
+	done = TRUE;
     }
+
     return done;
 }
 
@@ -291,10 +322,18 @@ typedef struct {
     int code;
 } CODENAME;
 
+#define ASCII_NAME(name,code)  { #name, code }
 #define CURSES_NAME(upper) { #upper, KEY_ ## upper }
-#define COUNT_CURSES  sizeof(curses_names)/sizeof(curses_names[0])
+#define COUNT_CURSES  TableSize(curses_names)
 static const CODENAME curses_names[] =
 {
+    ASCII_NAME(ESC, '\033'),
+    ASCII_NAME(CR, '\r'),
+    ASCII_NAME(LF, '\n'),
+    ASCII_NAME(FF, '\f'),
+    ASCII_NAME(TAB, '\t'),
+    ASCII_NAME(DEL, '\177'),
+
     CURSES_NAME(DOWN),
     CURSES_NAME(UP),
     CURSES_NAME(LEFT),
@@ -384,7 +423,7 @@ static const CODENAME curses_names[] =
 };
 
 #define DIALOG_NAME(upper) { #upper, DLGK_ ## upper }
-#define COUNT_DIALOG  sizeof(dialog_names)/sizeof(dialog_names[0])
+#define COUNT_DIALOG  TableSize(dialog_names)
 static const CODENAME dialog_names[] =
 {
     DIALOG_NAME(OK),
@@ -404,6 +443,10 @@ static const CODENAME dialog_names[] =
     DIALOG_NAME(FIELD_LAST),
     DIALOG_NAME(FIELD_NEXT),
     DIALOG_NAME(FIELD_PREV),
+    DIALOG_NAME(FORM_FIRST),
+    DIALOG_NAME(FORM_LAST),
+    DIALOG_NAME(FORM_NEXT),
+    DIALOG_NAME(FORM_PREV),
     DIALOG_NAME(GRID_UP),
     DIALOG_NAME(GRID_DOWN),
     DIALOG_NAME(GRID_LEFT),
@@ -414,13 +457,35 @@ static const CODENAME dialog_names[] =
     DIALOG_NAME(ENTER),
     DIALOG_NAME(BEGIN),
     DIALOG_NAME(FINAL),
-    DIALOG_NAME(SELECT)
+    DIALOG_NAME(SELECT),
+    DIALOG_NAME(HELPFILE),
+    DIALOG_NAME(TRACE),
+    DIALOG_NAME(TOGGLE)
 };
 
+#define MAP2(letter,actual) { letter, actual }
+
+static const struct {
+    int letter;
+    int actual;
+} escaped_letters[] = {
+
+    MAP2('a', DLG_CTRL('G')),
+	MAP2('b', DLG_CTRL('H')),
+	MAP2('f', DLG_CTRL('L')),
+	MAP2('n', DLG_CTRL('J')),
+	MAP2('r', DLG_CTRL('M')),
+	MAP2('s', CHR_SPACE),
+	MAP2('t', DLG_CTRL('I')),
+	MAP2('\\', '\\'),
+};
+
+#undef MAP2
+
 static char *
 skip_white(char *s)
 {
-    while (*s != '\0' && isspace(CharOf(*s)))
+    while (*s != '\0' && isspace(UCH(*s)))
 	++s;
     return s;
 }
@@ -428,7 +493,7 @@ skip_white(char *s)
 static char *
 skip_black(char *s)
 {
-    while (*s != '\0' && !isspace(CharOf(*s)))
+    while (*s != '\0' && !isspace(UCH(*s)))
 	++s;
     return s;
 }
@@ -468,9 +533,9 @@ compare_bindings(LIST_BINDINGS * a, LIST_BINDINGS * b)
     if (a->win == b->win) {
 	if (!strcmp(a->name, b->name)) {
 	    result = a->binding[0].curses_key - b->binding[0].curses_key;
-	} else if (!strcmp(b->name, "*")) {
+	} else if (!strcmp(b->name, WILDNAME)) {
 	    result = -1;
-	} else if (!strcmp(a->name, "*")) {
+	} else if (!strcmp(a->name, WILDNAME)) {
 	    result = 1;
 	} else {
 	    result = dlg_strcmp(a->name, b->name);
@@ -492,8 +557,8 @@ compare_bindings(LIST_BINDINGS * a, LIST_BINDINGS * b)
 static DLG_KEYS_BINDING *
 make_binding(char *widget, int curses_key, int is_function, int dialog_key)
 {
-    LIST_BINDINGS *entry;
-    DLG_KEYS_BINDING *data;
+    LIST_BINDINGS *entry = 0;
+    DLG_KEYS_BINDING *data = 0;
     char *name;
     LIST_BINDINGS *p, *q;
     DLG_KEYS_BINDING *result = find_binding(widget, curses_key);
@@ -526,19 +591,67 @@ make_binding(char *widget, int curses_key, int is_function, int dialog_key)
 	    entry->link = p;
 	}
 	result = data;
+    } else if (entry != 0) {
+	free(entry);
+	if (data)
+	    free(data);
     }
 
     return result;
 }
 
+static int
+decode_escaped(char **string)
+{
+    unsigned n;
+    int result = 0;
+
+    if (IsOctal(**string)) {
+	int limit = 3;
+	while (limit-- > 0 && IsOctal(**string)) {
+	    int ch = (**string);
+	    *string += 1;
+	    result = (result << 3) | (ch - '0');
+	}
+    } else {
+	for (n = 0; n < TableSize(escaped_letters); ++n) {
+	    if (**string == escaped_letters[n].letter) {
+		*string += 1;
+		result = escaped_letters[n].actual;
+		break;
+	    }
+	}
+    }
+    return result;
+}
+
+static char *
+encode_escaped(int value)
+{
+    static char result[80];
+    unsigned n;
+    bool found = FALSE;
+    for (n = 0; n < TableSize(escaped_letters); ++n) {
+	if (value == escaped_letters[n].actual) {
+	    found = TRUE;
+	    sprintf(result, "%c", escaped_letters[n].letter);
+	    break;
+	}
+    }
+    if (!found) {
+	sprintf(result, "%03o", value & 0xff);
+    }
+    return result;
+}
+
 /*
- * Parse the parameters of the "bindkeys" configuration-file entry.  This
+ * Parse the parameters of the "bindkey" configuration-file entry.  This
  * expects widget name which may be "*", followed by curses key definition and
  * then dialog key definition.
  *
  * The curses key "should" be one of the names (ignoring case) from
  * curses_names[], but may also be a single control character (prefix "^" or
- * "~" depending on whether it is C0 or C1), or an escaped single character. 
+ * "~" depending on whether it is C0 or C1), or an escaped single character.
  * Binding a printable character with dialog is possible but not useful.
  *
  * The dialog key must be one of the names from dialog_names[].
@@ -564,12 +677,13 @@ dlg_parse_bindkey(char *params)
     p = skip_black(p);
     if (p != widget && *p != '\0') {
 	*p++ = '\0';
+	p = skip_white(p);
 	q = p;
 	while (*p != '\0' && curses_key < 0) {
 	    if (escaped) {
 		escaped = FALSE;
-		curses_key = *p;
-	    } else if (*p == '\\') {
+		curses_key = decode_escaped(&p);
+	    } else if (*p == CHR_BACKSLASH) {
 		escaped = TRUE;
 	    } else if (modified) {
 		if (*p == '?') {
@@ -585,12 +699,12 @@ dlg_parse_bindkey(char *params)
 		modified = *p;
 	    } else if (*p == '~') {
 		modified = *p;
-	    } else if (isspace(CharOf(*p))) {
+	    } else if (isspace(UCH(*p))) {
 		break;
 	    }
 	    ++p;
 	}
-	if (!isspace(CharOf(*p))) {
+	if (!isspace(UCH(*p))) {
 	    ;
 	} else {
 	    *p++ = '\0';
@@ -605,7 +719,7 @@ dlg_parse_bindkey(char *params)
 		    for (xx = 0; xx < COUNT_CURSES; ++xx) {
 			if (!dlg_strcmp(curses_names[xx].name, q)) {
 			    curses_key = curses_names[xx].code;
-			    is_function = TRUE;
+			    is_function = (curses_key >= KEY_MIN);
 			    break;
 			}
 		    }
@@ -646,6 +760,12 @@ dump_curses_key(FILE *fp, int curses_key)
 	    }
 	}
 	if (!found) {
+#ifdef KEY_MOUSE
+	    if (is_DLGK_MOUSE(curses_key)) {
+		fprintf(fp, "MOUSE-");
+		dump_curses_key(fp, curses_key - M_EVENT);
+	    } else
+#endif
 	    if (curses_key >= KEY_F(0)) {
 		fprintf(fp, "F%d", curses_key - KEY_F(0));
 	    } else {
@@ -660,8 +780,12 @@ dump_curses_key(FILE *fp, int curses_key)
 	fprintf(fp, "~%c", curses_key - 64);
     } else if (curses_key == 255) {
 	fprintf(fp, "~?");
+    } else if (curses_key > 32 &&
+	       curses_key < 127 &&
+	       curses_key != CHR_BACKSLASH) {
+	fprintf(fp, "%c", curses_key);
     } else {
-	fprintf(fp, "\\%c", curses_key);
+	fprintf(fp, "%c%s", CHR_BACKSLASH, encode_escaped(curses_key));
     }
 }
 
@@ -692,30 +816,53 @@ dump_one_binding(FILE *fp, const char *widget, DLG_KEYS_BINDING * binding)
     fputc('\n', fp);
 }
 
+/*
+ * Dump bindings for the given window.  If it is a null, then this dumps the
+ * initial bindings which were loaded from the rc-file that are used as
+ * overall defaults.
+ */
 void
-dlg_dump_keys(FILE *fp)
+dlg_dump_window_keys(FILE *fp, WINDOW *win)
 {
-    LIST_BINDINGS *p;
-    const char *last = "";
-    unsigned n;
-    unsigned count = 0;
+    if (fp != 0) {
+	LIST_BINDINGS *p;
+	DLG_KEYS_BINDING *q;
+	const char *last = "";
 
-    for (p = all_bindings; p != 0; p = p->link) {
-	if (p->win == 0) {
-	    ++count;
-	}
-    }
-    if (count != 0) {
-	for (p = all_bindings, n = 0; p != 0; p = p->link) {
-	    if (p->win == 0) {
+	for (p = all_bindings; p != 0; p = p->link) {
+	    if (p->win == win) {
 		if (dlg_strcmp(last, p->name)) {
 		    fprintf(fp, "\n# key bindings for %s widgets\n",
-			    !strcmp(p->name, "*") ? "all" : p->name);
+			    !strcmp(p->name, WILDNAME) ? "all" : p->name);
 		    last = p->name;
 		}
-		dump_one_binding(fp, p->name, p->binding);
+		for (q = p->binding; q->is_function_key >= 0; ++q) {
+		    dump_one_binding(fp, p->name, q);
+		}
 	    }
 	}
     }
 }
+
+/*
+ * Dump all of the bindings which are not specific to a given widget, i.e.,
+ * the "win" member is null.
+ */
+void
+dlg_dump_keys(FILE *fp)
+{
+    if (fp != 0) {
+	LIST_BINDINGS *p;
+	unsigned count = 0;
+
+	for (p = all_bindings; p != 0; p = p->link) {
+	    if (p->win == 0) {
+		++count;
+	    }
+	}
+	if (count != 0) {
+	    dlg_dump_window_keys(fp, 0);
+	}
+    }
+}
 #endif /* HAVE_RC_FILE */
