@@ -1,9 +1,9 @@
 /*
- * $Id: mouse.c,v 1.18 2007/02/22 21:51:38 tom Exp $
+ * $Id: mouse.c,v 1.23 2016/02/03 22:49:55 tom Exp $
  *
  * mouse.c -- mouse support for dialog
  *
- * Copyright 2002-2006,2007	Thomas E. Dickey
+ * Copyright 2002-2015,2016	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -26,7 +26,7 @@
 
 #if USE_MOUSE
 
-static int basex, basey;
+static int basex, basey, basecode;
 
 static mseRegion *regionList = NULL;
 
@@ -51,6 +51,13 @@ dlg_mouse_setbase(int x, int y)
     basey = y;
 }
 
+void
+dlg_mouse_setcode(int code)
+{
+    basecode = code;
+    dlg_trace_msg("# mouse_setcode %d\n", code);
+}
+
 void
 dlg_mouse_mkbigregion(int y, int x,
 		      int height, int width,
@@ -79,22 +86,35 @@ dlg_mouse_mkregion(int y, int x, int height, int width, int code)
 {
     mseRegion *butPtr;
 
-    if ((butPtr = find_region_by_code(code)) == 0) {
-	butPtr = dlg_malloc(mseRegion, 1);
+    if ((butPtr = find_region_by_code(basecode + code)) == 0) {
+	butPtr = dlg_calloc(mseRegion, 1);
 	assert_ptr(butPtr, "dlg_mouse_mkregion");
 	butPtr->next = regionList;
 	regionList = butPtr;
     }
-    if (butPtr != 0) {
-	butPtr->mode = -1;
-	butPtr->step_x = 0;
-	butPtr->step_y = 0;
-	butPtr->y = basey + y;
-	butPtr->Y = basey + y + height;
-	butPtr->x = basex + x;
-	butPtr->X = basex + x + width;
-	butPtr->code = code;
+
+    if ((butPtr->mode != -1) ||
+	(butPtr->step_x != 0) ||
+	(butPtr->step_y != 0) ||
+	(butPtr->y != (basey + y)) ||
+	(butPtr->Y != (basey + y + height)) ||
+	(butPtr->x != (basex + x)) ||
+	(butPtr->X != (basex + x + width)) ||
+	(butPtr->code != basecode + code)) {
+	dlg_trace_msg("# mouse_mkregion %d,%d %dx%d %d (%d)\n",
+		      y, x, height, width,
+		      butPtr->code, code);
     }
+
+    butPtr->mode = -1;
+    butPtr->step_x = 0;
+    butPtr->step_y = 0;
+    butPtr->y = basey + y;
+    butPtr->Y = basey + y + height;
+    butPtr->x = basex + x;
+    butPtr->X = basex + x + width;
+    butPtr->code = basecode + code;
+
     return butPtr;
 }
 
@@ -105,12 +125,15 @@ any_mouse_region(int y, int x, int small)
     mseRegion *butPtr;
 
     for (butPtr = regionList; butPtr; butPtr = butPtr->next) {
-	if (small ^ (butPtr->code >= 0))
+	if (small ^ (butPtr->code >= 0)) {
 	    continue;
-	if (y < butPtr->y || y >= butPtr->Y)
+	}
+	if (y < butPtr->y || y >= butPtr->Y) {
 	    continue;
-	if (x < butPtr->x || x >= butPtr->X)
+	}
+	if (x < butPtr->x || x >= butPtr->X) {
 	    continue;
+	}
 	break;			/* found */
     }
     return butPtr;
